import { arbres as importedData } from './forest.js';
import { generateTree, getColorByAccuracy, calculateBoundingCircle, calculateTreeXPosition, setBubble} from './functions.js';

// Définition des dimensions de la visualisation.
const width = window.innerWidth;
const height = window.innerHeight;
const margin = {top:20, right:20, bottom:70, left:20};

// Création du canevas SVG.
const svg = d3.select("#arbre_svg")
  .attr("width", width)
  .attr("height", height);

// Ajout d'un groupe (g) pour contenir les éléments de la visualisation.
let g = svg.append("g")
  .attr("transform", `translate(${margin.left},${margin.top})`);

// Variables pour ajuster les positions et tailles des arbres.
let xOffset = 0;
let maxHeight = 0;

// Facteur d'espacement vertical pour ajuster l'espace entre les arbres
const verticalSpacingFactor = 1.5;
// Facteur d'espacement pour ajuster l'espace entre les arbres
const spacingFactor = 1.2;

const axisMarginBottom = 150;

// Conversion des données en arbres hiérarchiques.
const group1 = [];
const group2 = [];
const group3 = [];

let totalWidth = width*0.8;

// Mettre à jour la largeur totale pour inclure le facteur d'espacement
totalWidth = totalWidth * spacingFactor;

// Création d'une échelle pour l'axe de l'accuracy.
const accuracyScale = d3.scaleLinear()
  .domain([1, 0.8, 0.6, 0])
  .range([0, (totalWidth - margin.left - margin.right) / 3, (totalWidth - margin.left - margin.right) * 2 / 3, totalWidth - margin.left - margin.right]);

// Configuration de la disposition de l'arbre.
const treeLayout = d3.tree().size(
  [0.3 * (height - margin.top - margin.bottom), 
  0.3 * (width - margin.left - margin.right)/3.5]);

// Variables pour ajuster les positions des arbres.
let yOffsetGroup1 = [];
let yOffsetGroup2 = [];
let yOffsetGroup3 = [];

importedData.forEach((treeData, i) => {
  const tree = generateTree(treeData, i + 1, treeLayout);
  if (tree.data.accuracy >= 0.8) {
    group1.push(tree);
  } else if (tree.data.accuracy >= 0.6) {
    group2.push(tree);
  } else {
    group3.push(tree);
  }
});

// Tri des arbres par accuracy dans l'ordre décroissant dans chaque groupe
group1.sort((a, b) => b.data.accuracy - a.data.accuracy);
group2.sort((a, b) => b.data.accuracy - a.data.accuracy);
group3.sort((a, b) => b.data.accuracy - a.data.accuracy);

// Concaténation des groupes triés
const data = group1.concat(group2).concat(group3);

const boundingCircles = data.map(tree => calculateBoundingCircle(tree));

// Créer un objet pour mapper les groupes aux plages de précision
const accuracyRanges = {
  "Tous les groupes": {min: 0, max: 1},
  "Groupe 1": {min: 0.8, max: 1},
  "Groupe 2": {min: 0.6, max: 0.8},
  "Groupe 3": {min: 0, max: 0.6}
};


// Fonction pour basculer la classe "toggled" sur les nœuds ayant le même nom.
export function toggleSameNodes(d) {
  // Trouver tous les nœuds portant le même nom dans tous les arbres
  const sameNodes = data.flatMap(tree => tree.descendants())
    .filter(node => node.data.name === d.data.name);

  // Basculer la classe "toggled" sur les nœuds trouvés
  svg.selectAll(".node")
    .filter(node => sameNodes.includes(node))
    .classed("toggled", sameNodes[0].data.toggled ? false : true);
}

//****************************************************
//****************************************************
function prepareDataForVisualization(selectedFeature, selectedGroup, selectedFeatures) {
  // Filtrer les données en fonction de la caractéristique sélectionnée.
  let filteredData;
  if (selectedFeature && selectedFeature !== "Toutes les caractéristiques") {
    filteredData = data.filter(tree =>
      tree.descendants().some(node => node.data.name === selectedFeature)
    );
  } else {
    filteredData = data; // Si aucune caractéristique n'est sélectionnée ou si "Toutes les caractéristiques" est sélectionné, utiliser toutes les données.
  }

  // Filtrer les données en fonction des caractéristiques sélectionnées avec les boutons 'Meilleures' et 'Pires'.
  if (selectedFeatures && selectedFeatures.length > 0) {
    filteredData = filteredData.filter(tree =>
      tree.descendants().some(node => selectedFeatures.includes(node.data.name))
    );
  }

  // Filtrer les données en fonction du groupe sélectionné.
  let selectedAllGroups = false;
  if (selectedGroup && selectedGroup !== "Tous les groupes") {
    const range = accuracyRanges[selectedGroup];
    if (range) {
      filteredData = filteredData.filter(tree =>
        tree.data.accuracy >= range.min && tree.data.accuracy < range.max
      );
    } else {
      console.error(`Unknown group: ${selectedGroup}`);
    }
  } else {
    selectedAllGroups = true;
  }

  return { filteredData, selectedAllGroups };
}

//////////////////////////////////////////////////////////////////////////////////

let classes = []; // ajouter un tableau pour stocker les classes
let caractéristiques = [];

importedData.forEach(arbre => {
  const extractChildrenNames = (node) => {
    // si le node est une feuille (classe), ajouter à l'array de classes, sinon à caractéristiques
    if (!node.children) {
      classes.push(node.name);
    } else {
      caractéristiques.push(node.name);
      node.children.forEach(child => extractChildrenNames(child));
    }
  };
  if (arbre.children) {
    arbre.children.forEach(child => extractChildrenNames(child));
  }
});

classes = [...new Set(classes)];

// Supprimer les doublons et les trier
caractéristiques = [...new Set(caractéristiques)];

classes.sort();

// Trier les caractéristiques et les groupes
caractéristiques.sort((a, b) => {
  const numA = parseInt(a.slice(1)); 
  const numB = parseInt(b.slice(1)); 
  return numA - numB; 
});

caractéristiques.unshift("Toutes les caractéristiques");

//Légende des classes
// créer une échelle de couleur
let couleur = d3.scaleOrdinal(d3.schemeCategory10);
// assigner une couleur à chaque classe
classes.forEach((classe, i) => {
  couleur(classe);
});

/////////////////////////////////////////////////////////////////////////

function generateAllGroupsVisualization(filteredData, selectedFeature) {

  // Réinitialiser les variables de positionnement
  let yOffsetGroup1 = [];
  let yOffsetGroup2 = [];
  let yOffsetGroup3 = [];


  // Boucle sur chaque arbre pour créer la visualisation.
  filteredData.forEach((tree, index) => {
    const treeWidths = tree.descendants().map(d => d.x);
    const maxTreeWidth = Math.max(...treeWidths);
    const treeHeights = tree.descendants().map(d => d.y);
    const maxTreeHeight = Math.max(...treeHeights);


    // Calcul de la position X de l'arbre, change si un groupe spécifique est sélectionné.
    let treeXPosition;
    treeXPosition = calculateTreeXPosition(tree.data.accuracy, totalWidth);
  

    let yOffset;
    const boundingCircleRadius = boundingCircles[index].radius;
    const accumulatedRadius = (group) => group.reduce((acc, radius) => acc + radius, 0);

    let groupSpacing = -60; 

    if (tree.data.accuracy >= 0.8) {
      yOffset = accumulatedRadius(yOffsetGroup1) + yOffsetGroup1.length * (maxHeight + margin.bottom + groupSpacing) + axisMarginBottom;
      yOffsetGroup1.push(boundingCircleRadius);
    } else if (tree.data.accuracy >= 0.6) {
      yOffset = accumulatedRadius(yOffsetGroup2) + yOffsetGroup2.length * (maxHeight + margin.bottom + groupSpacing) + axisMarginBottom;
      yOffsetGroup2.push(boundingCircleRadius);
    } else {
      yOffset = accumulatedRadius(yOffsetGroup3) + yOffsetGroup3.length * (maxHeight + margin.bottom + groupSpacing) + axisMarginBottom;
      yOffsetGroup3.push(boundingCircleRadius);
    }

    // Création d'un groupe (g) pour chaque arbre.
    const treeGroup = g.append("g")
      .attr("transform", `translate(${treeXPosition - boundingCircles[index].centerX}, ${yOffset})`);

    // Dessiner le cercle circonscrit pour chaque arbre en utilisant la couleur de la racine.
    const boundingCircle = boundingCircles[index];
    const rootColor = getColorByAccuracy(tree.data.accuracy);

    treeGroup.append("circle")
      .attr("class", "bounding-circle")
      .attr("cx", boundingCircle.centerX)
      .attr("cy", boundingCircle.centerY)
      .attr("r", boundingCircle.radius)
      .attr("stroke", rootColor) // Utilise la couleur de la racine pour le contour du cercle.
      .attr("fill", rootColor) // Utilise la couleur de la racine pour le remplissage du cercle.
      .attr("opacity", 0.3); // Ajoute une opacité pour rendre le cercle semi  transparent.

      treeGroup.append("circle")
        .attr("class", "bounding-circle")
        .attr("cx", boundingCircle.centerX)
        .attr("cy", boundingCircle.centerY)
        .attr("r", boundingCircle.radius)
        .attr("stroke", "black")
        .attr("fill", "none");
  
      treeGroup.append("text")
        .attr("class", "accuracy-label")
        .attr("x", boundingCircle.centerX)
        .attr("y", 0) // Ajuste la position verticale des étiquettes
        .attr("text-anchor", "middle")
        .text(`${tree.data.accuracy.toFixed(2)}`)
        .attr("font-family", "Arial")
        .attr("font-size", "12px")
        .attr("fill", "black");
  
      // Dessiner les nœuds de chaque arbre (sauf la racine)
      const nodesData = tree.descendants().filter(d => d.parent);
      const nodes = treeGroup.selectAll(".node")
        .data(nodesData, d => d.data.name);
  
      const nodesEnter = nodes.enter()
        .append("g")
        .attr("class", "node")
        .attr("transform", (d) => `translate(${d.x},${d.y})`)
        .on("mouseover", function(d) {
          tooltip.transition()
            .duration(200)
            .style("opacity", .9);
          tooltip.html(d.data.name)
            .style("left", (d3.event.pageX) + "px")
            .style("top", (d3.event.pageY - 28) + "px");
        })
        .on("mouseout", function(d) {
          tooltip.transition()
            .duration(500)
            .style("opacity", 0);
        })
        .on("click", function(d) {
          const node = d3.select(this);
          const isToggled = node.classed("toggled");
  
          // Supprimer la classe "toggled" de tous les nœuds de toutes les arborescences
          svg.selectAll(".node").classed("toggled", false);
  
          // Basculer la classe "toggled" sur le nœud cliqué et les mêmes nœuds
          node.classed("toggled", !isToggled);
          toggleSameNodes(d);
          
          if (d.children) {
            selectedFeature = d.data.name;
            // Mettre à jour l'affichage de la fonctionnalité sélectionnée
            d3.select('#selected-feature-display').text(`Caractéristique: ${selectedFeature}`);
          }  
          else {
            // Effacer le texte si le nœud cliqué est une feuille
            d3.select('#selected-feature-display').text('');}
        });
  
        nodesEnter.append("circle")
          .attr("r", 8)
          .style('fill', d => {
              // Si le nœud est une feuille et que son nom correspond à une classe, on utilise la couleur correspondante
              if (!d.children && classes.includes(d.data.name)) {
                  return couleur(d.data.name);
              }
              // Sinon, on colorie le nœud en noir ou en vert selon que le nom du nœud correspond à la caractéristique sélectionnée
              else {
                  return d.data.name === selectedFeature ? '#90EE90' : '#black';
              }
          }) 
          .style('stroke', d => {
            // Si le nœud est une feuille, on lui donne un bord noir
            return (!d.children) ? 'black' : 'none';
          })
          .style('stroke-width', d => {
              // Si le nœud est une feuille, on lui donne un bord d'épaisseur 2
              return (!d.children) ? '2' : 'none';
          })
          .attr("data-name", d => d.data.name);
  
      nodes.exit().remove();
  
      // Ajout d'une infobulle pour afficher le nom du nœud.
      const tooltip = d3.select("body").append("div")
        .attr("class", "tooltip")
        .style("opacity", 0);
  
      // Dessiner les liens entre les nœuds.
      const linksData = tree.links();
      const filteredLinksData = linksData.filter(d => d.source.parent); // Exclut les liens où la source est la racine
      const links = treeGroup.selectAll(".link")
        .data(filteredLinksData, d => d.source.data.name + '-' + d.target.data.name);  

        const linksEnter = links.enter()
          .append("path")
          .attr("class", "link")
          .attr("d", d3.linkVertical().x(d => d.x).y(d => d.y))
          .attr("stroke", "black")
          .attr("fill", "none");
    
        links.exit().remove();
    
        // Mise à jour des variables de positionnement et de taille.
        xOffset += maxTreeWidth + margin.right;
        maxHeight = Math.max(maxHeight, maxTreeHeight);
      });
    
      // Ajustement de la largeur et de la hauteur de la zone de dessin si nécessaire.
      const maxGroupLength = Math.max(yOffsetGroup1.length, yOffsetGroup2.length, yOffsetGroup3.length);
      const totalHeight = maxGroupLength * (maxHeight + margin.bottom) * verticalSpacingFactor + margin.top + margin.bottom;
      if (totalWidth > width) {
          svg.attr("width", totalWidth);
      }
      svg.attr("height", Math.max(height, totalHeight) + axisMarginBottom);

      // Création de l'axe de l'accuracy.
      const accuracyAxis = d3.axisBottom(accuracyScale)
        .tickValues([1, 0.8, 0.6, 0]) // Valeurs spécifiques qu'on souhaite afficher
        .tickFormat(d3.format(".0%"));

      // Ajout de l'axe de l'accuracy.
      svg.append("g")
        .attr("class", "accuracy-axis")
        .attr("transform", `translate(10, 140)`) //Position de l'axe d'accuracy
        .call(accuracyAxis);

      const axeLabelPos = 6.5;
      // Ajout du libellé "Accuracy" en haut à gauche de l'axe
      svg.append("text")
      .attr("class", "axis-label")
      .attr("x", margin.left)
      .attr("y", margin.top * axeLabelPos)
      .attr("text-anchor", "start")
      .text("Accuracy");

      // Ajout des libellés des groupes au-dessus de l'axe
      svg.append("text")
      .attr("class", "group-label")
      .attr("x", (totalWidth - margin.left - margin.right) / 6) // Position x pour le libellé "Groupe 1"
      .attr("y", margin.top * axeLabelPos) // Position y pour tous les libellés
      .attr("text-anchor", "middle")
      .text("Groupe 1");

      svg.append("text")
      .attr("class", "group-label")
      .attr("x", (totalWidth - margin.left - margin.right) / 2) // Position x pour le libellé "Groupe 2"
      .attr("y", margin.top * axeLabelPos) // Position y pour tous les libellés
      .attr("text-anchor", "middle")
      .text("Groupe 2");

      svg.append("text")
      .attr("class", "group-label")
      .attr("x", (totalWidth - margin.left - margin.right) * 5 / 6) // Position x pour le libellé "Groupe 3"
      .attr("y", margin.top * axeLabelPos) // Position y pour tous les libellés
      .attr("text-anchor", "middle")
      .text("Groupe 3");

      // Mise à jour la position X du titre de l'axe de l'accuracy.
      svg.select(".accuracy-axis-title")
        .attr("x", totalWidth / 2);
    
}
/////////////////////////////////////////////////////////////////////////////////////

function generateSpecificGroupVisualization(filteredData, selectedFeature) {

  d3.select(".accuracy-axis").remove();
  d3.select(".axis-label").remove();
  d3.selectAll(".group-label").remove();
  
  // Trier les données par accuracy en ordre décroissant pour le groupe spécifique sélectionné.
  filteredData.sort((a, b) => b.data.accuracy - a.data.accuracy);

  let xOffset = 50; // Cette variable déterminera la position X de chaque arbre.
  
  filteredData.forEach((tree, index) => {
    const treeWidths = tree.descendants().map(d => d.x);
    const maxTreeWidth = Math.max(...treeWidths);
    const treeHeights = tree.descendants().map(d => d.y);
    const maxTreeHeight = Math.max(...treeHeights);
    
    // Calcul de la position X de l'arbre.
    let treeXPosition = xOffset;
    
    // Créer un groupe pour chaque arbre.
    const treeGroup = g.append("g")
      .attr("transform", `translate(${treeXPosition}, ${100})`);

    // Dessiner le cercle circonscrit pour chaque arbre en utilisant la couleur de la racine.
    const boundingCircle = boundingCircles[index];
    const rootColor = getColorByAccuracy(tree.data.accuracy);

  treeGroup.append("circle")
    .attr("class", "bounding-circle")
    .attr("cx", boundingCircle.centerX)
    .attr("cy", boundingCircle.centerY)
    .attr("r", boundingCircle.radius)
    .attr("stroke", rootColor) // Utilise la couleur de la racine pour le contour du cercle.
    .attr("fill", rootColor) // Utilise la couleur de la racine pour le remplissage du cercle.
    .attr("opacity", 0.3); // Ajoute une opacité pour rendre le cercle semi  transparent.

    treeGroup.append("circle")
      .attr("class", "bounding-circle")
      .attr("cx", boundingCircle.centerX)
      .attr("cy", boundingCircle.centerY)
      .attr("r", boundingCircle.radius)
      .attr("stroke", "black")
      .attr("fill", "none");

    treeGroup.append("text")
      .attr("class", "accuracy-label")
      .attr("x", boundingCircle.centerX)
      .attr("y", 0) // Ajuste la position verticale des étiquettes
      .attr("text-anchor", "middle")
      .text(`${tree.data.accuracy.toFixed(2)}`)
      .attr("font-family", "Arial")
      .attr("font-size", "12px")
      .attr("fill", "black");

    // Dessiner les nœuds de chaque arbre (sauf la racine)
    const nodesData = tree.descendants().filter(d => d.parent);
    const nodes = treeGroup.selectAll(".node")
      .data(nodesData, d => d.data.name);  

    const nodesEnter = nodes.enter()
      .append("g")
      .attr("class", "node")
      .attr("transform", (d) => `translate(${d.x},${d.y})`)
      .on("mouseover", function(d) {
        tooltip.transition()
          .duration(200)
          .style("opacity", .9);
        tooltip.html(d.data.name)
          .style("left", (d3.event.pageX) + "px")
          .style("top", (d3.event.pageY - 28) + "px");
      })
      .on("mouseout", function(d) {
        tooltip.transition()
          .duration(500)
          .style("opacity", 0);
      })
      .on("click", function(d) {
        const node = d3.select(this);
        const isToggled = node.classed("toggled");

        // Supprimer la classe "toggled" de tous les nœuds de toutes les arborescences
        svg.selectAll(".node").classed("toggled", false);

        // Basculer la classe "toggled" sur le nœud cliqué et les mêmes nœuds
        node.classed("toggled", !isToggled);
        toggleSameNodes(d);
        
        if (d.children) {
          selectedFeature = d.data.name;
          // Mettre à jour l'affichage de la fonctionnalité sélectionnée
          d3.select('#selected-feature-display').text(`Caractéristique: ${selectedFeature}`);
        }  
        else {
          // Effacer le texte si le nœud cliqué est une feuille
          d3.select('#selected-feature-display').text('');}
      });

      nodesEnter.append("circle")
      .attr("r", 8)
      .style('fill', d => {
          // Si le nœud est une feuille et que son nom correspond à une classe, on utilise la couleur correspondante
          if (!d.children && classes.includes(d.data.name)) {
              return couleur(d.data.name);
          }
          // Sinon, on colorie le nœud en noir ou en vert selon que le nom du nœud correspond à la caractéristique sélectionnée
          else {
              return d.data.name === selectedFeature ? '#90EE90' : '#black';
          }
      }) 
      .style('stroke', d => {
        // Si le nœud est une feuille, bord noir
        return (!d.children) ? 'black' : 'none';
      })
      .style('stroke-width', d => {
          // Si le nœud est une feuille, bord d'épaisseur 2
          return (!d.children) ? '2' : 'none';
      })
      .attr("data-name", d => d.data.name)

    nodes.exit().remove();

    // Ajout d'une infobulle pour afficher le nom du nœud.
    const tooltip = d3.select("body").append("div")
      .attr("class", "tooltip")
      .style("opacity", 0);

    // Dessiner les liens entre les nœuds.
    const linksData = tree.links();
    const filteredLinksData = linksData.filter(d => d.source.parent); // Exclut les liens où la source est la racine
    const links = treeGroup.selectAll(".link")
      .data(filteredLinksData, d => d.source.data.name + '-' + d.target.data.name); 

      const linksEnter = links.enter()
        .append("path")
        .attr("class", "link")
        .attr("d", d3.linkVertical().x(d => d.x).y(d => d.y))
        .attr("stroke", "black")
        .attr("fill", "none");
  
      links.exit().remove();
  
      // Mise à jour des variables de positionnement et de taille.
      let treeSpacing = 50;  
      xOffset += maxTreeWidth + margin.right + treeSpacing;
      svg.attr("width", Math.max(svg.attr("width"), xOffset));
      maxHeight = Math.max(maxHeight, maxTreeHeight);
    });
  
    // Ajustement de la largeur et de la hauteur de la zone de dessin si nécessaire.
    const maxGroupLength = Math.max(yOffsetGroup1.length, yOffsetGroup2.length, yOffsetGroup3.length);
    const totalHeight = maxGroupLength * (maxHeight + margin.bottom) * verticalSpacingFactor + margin.top + margin.bottom;
    if (totalWidth > width) {
        svg.attr("width", totalWidth);
    }
    svg.attr("height", Math.max(height, totalHeight) + axisMarginBottom);
    
}
/////////////////////////////////////////////////////////////////////////////////////

function generateVisualization(selectedFeature, selectedGroup, selectedFeatures) {
  const { filteredData, selectedAllGroups } = prepareDataForVisualization(selectedFeature, selectedGroup, selectedFeatures);
  
  // Effacer l'ancienne visualisation
  g.selectAll("*").remove();

  if (selectedAllGroups) {
    generateAllGroupsVisualization(filteredData,selectedFeature);
  } else {
    generateSpecificGroupVisualization(filteredData,selectedFeature);
  }
}

///////////////////////////////////////////////////////////////////////////////////

//****************************************************
//****************************************************
// Générer la visualisation initiale avec toutes les caractéristiques et tous les groupes.
// Initialiser selectedFeatures avec toutes les caractéristiques
let selectedFeature = "Toutes les caractéristiques";
let selectedFeatures = caractéristiques; 
let selectedGroup = "Tous les groupes";
generateVisualization(selectedFeature, selectedGroup, selectedFeatures); 


// créer la légende, on place notre légende en haut à gauche en utilisant le style css
let legende = d3.select('body').append('div')
  .attr('class', 'legende')
 
// ajouter un élément pour chaque classe
classes.forEach((classe, i) => {
  legende.append('div')
    .attr('class', 'legende-item')
    .style('background-color', couleur(classe)) // assigner une couleur à chaque classe
    .text(classe);
});
// Les groupes sont définis manuellement en fonction de la précision
let groupes = ["Tous les groupes", "Groupe 1", "Groupe 2", "Groupe 3"];

// Ajouter les caractéristiques à la première liste déroulante
const selectElementFeature = d3.select('#FeatureSelect');
const optionsFeature = selectElementFeature.selectAll('option')
  .data(caractéristiques, d => d); 

optionsFeature.enter()
  .append('option')
  .attr('value', d => d)
  .text(d => d);

optionsFeature.exit().remove();

// Initialiser Select2 après avoir ajouté de nouvelles options
$('#FeatureSelect').select2();

$('#FeatureSelect').on('change', function() {
  selectedFeature = $(this).val();
  const selectedGroup = d3.select('#groupSelect').property('value');
  generateVisualization(selectedFeature, selectedGroup, selectedFeatures);
  d3.select('#selected-feature-display').text('');
});



// Ajouter les groupes à la deuxième liste déroulante
const selectElementGroup = d3.select('#groupSelect');
const optionsGroup = selectElementGroup.selectAll('option')
  .data(groupes, d => d); 

optionsGroup.enter()
  .append('option')
  .attr('value', d => d)
  .text(d => d);

optionsGroup.exit().remove();

selectElementGroup.on('change', function() {
  const selectedGroup = d3.select(this).property('value');
  const selectedFeature = d3.select('#FeatureSelect').property('value');
  generateVisualization(selectedFeature, selectedGroup, selectedFeatures);
});

//Losqu'on choisi une valeur sur le slider

let numFeaturesSlider; //pour stocker la valeur choisie 
let featureImportance; // pour stocker les indices des caractéristiques

fetch('feature_importances.json')
.then(response => response.json())
.then(data => {
  featureImportance = data;
    
    // définir la valeur max de l'input range
    numFeaturesSlider = document.getElementById('numFeatures');
    numFeaturesSlider.max = featureImportance.length;
    
    // Afficher la valeur max du curseur dans la console
    console.log('La valeur max du curseur est :', numFeaturesSlider.max);
    console.log('featureImportance = ', featureImportance);
    numFeaturesSlider.value = 0;  // initialize slider with a value of 0

    let bubble = document.querySelector(".bubble");  // initialize bubble variable
    setBubble(numFeaturesSlider, bubble);  // initialize bubble

    // Mettre à jour la bulle chaque fois que le curseur change
    numFeaturesSlider.addEventListener('input', () => {
        setBubble(numFeaturesSlider, bubble);
        // Log the new slider value
        console.log("La valeur actuelle du curseur est :", numFeaturesSlider.value);

        // Supprimer la classe 'button-selected' de tous les boutons
        bestButton.classList.remove('button-selected');
        worstButton.classList.remove('button-selected');
    });

})
.catch(error => console.error(error));

//lorsqu'on clique sur les bouttons "Meilleures" ou "Pires"
const bestButton = document.getElementById('bestButton');
const worstButton = document.getElementById('worstButton');

bestButton.addEventListener('click', function() {
  bestButton.classList.add('button-selected');
  bestButton.classList.remove('button-unselected');
  worstButton.classList.add('button-unselected');
  worstButton.classList.remove('button-selected');

  // Récupérer le nombre de caractéristiques à partir de la valeur du slider
  let numFeatures = numFeaturesSlider.value;
  console.log("Nombre de meilleures caractéristiques sélectionnées :", numFeatures);

  // On prend les 'numFeatures' premiers indices
  let selectedFeatures = featureImportance.slice(0, numFeatures);
  console.log("Tableau des meilleures caractéristiques sélectionnées :", selectedFeatures);

  // Générer la visualisation avec ces caractéristiques
  generateVisualization(selectedFeature, selectedGroup, selectedFeatures);
});


worstButton.addEventListener('click', function() {
  worstButton.classList.add('button-selected');
  worstButton.classList.remove('button-unselected');
  bestButton.classList.add('button-unselected');
  bestButton.classList.remove('button-selected');

  // Récupérer le nombre de caractéristiques à partir de la valeur du slider
  let numFeatures = numFeaturesSlider.value;
  console.log("Nombre de pires caractéristiques sélectionnées :", numFeatures);

  // On prend les 'numFeatures' derniers indices
  let selectedFeatures = featureImportance.slice(-numFeatures);
  console.log("Tableau des meilleures caractéristiques sélectionnées :", selectedFeatures);

  // Générer la visualisation avec ces caractéristiques
  generateVisualization(selectedFeature, selectedGroup, selectedFeatures);
});

//récupérer les valeurs du fichier feature_values dans un tableau
let featureImportanceValues = []; // Pour stocker les valeurs d'importance des caractéristiques
fetch('feature_values.json')
  .then(response => response.json())
  .then(data => {
    featureImportanceValues = data;

    // Votre code supplémentaire ici pour utiliser les valeurs d'importance des caractéristiques

    console.log('featureImportanceValues:', featureImportanceValues);
  })
  .catch(error => console.error(error));


// Ecouteur d'événement pour fermer le modal
let closeModalButton = document.getElementById('closeModalButton');
closeModalButton.addEventListener('click', function() {
  $('#featureModal').modal('hide');
});

// Bouton pour afficher le classement des caractéristiques 
let allButton = document.getElementById('allButton');
allButton.addEventListener('click', function() {
  let modalBody = '';
  featureImportance.forEach((feature, index) => {
    const featureValue = featureImportanceValues[index]; // Récupérer la valeur correspondante

    const formattedValue = Number(featureValue).toFixed(2); // Formater la valeur